@@ -3,13 +3,14 @@ package DateTime::Format::Strptime;
 use strict;
 use warnings;
 
-our $VERSION = '1.63';
+our $VERSION = '1.72';
 
 use Carp qw( carp croak );
 use DateTime 1.00;
-use DateTime::Locale 0.45;
-use DateTime::TimeZone 0.79;
-use Params::Validate 1.20 qw( validate SCALAR BOOLEAN OBJECT CODEREF );
+use DateTime::Locale 1.05;
+use DateTime::Format::Strptime::Types;
+use DateTime::TimeZone 2.09;
+use Params::ValidationCompiler qw( validation_for );
 use Try::Tiny;
 
 use Exporter qw( import );
@@ -20,42 +21,45 @@ use Package::DeprecationManager 0.15 -deprecations => {
 
 our @EXPORT_OK = qw( strftime strptime );
 
+## no critic (ValuesAndExpressions::ProhibitConstantPragma)
 use constant PERL_58 => $] < 5.010;
 
 {
-    my $spec = {
-        pattern => { type => SCALAR },
-        time_zone => {
-            type     => SCALAR | OBJECT,
-            optional => 1,
-        },
-        locale => {
-            type    => SCALAR | OBJECT,
-            default => DateTime::Locale->load('en'),
-        },
-        on_error => {
-            type      => SCALAR | CODEREF,
-            default   => 'undef',
-            callbacks => {
-                'valid on_error' => sub {
-                    return 1
-                        if $_[0]
-                        && ( ref $_[0] eq 'CODE'
-                        || $_[0] =~ /\A(?:croak|undef)\z/ );
-                    die
-                        q{The value supplied to on_error must be either 'croak', 'undef' or a code reference.};
-                },
+    my $en_locale = DateTime::Locale->load('en');
+
+    my $validator = validation_for(
+        params => {
+            pattern   => { type => t('NonEmptyStr') },
+            time_zone => {
+                type     => t('TimeZone'),
+                optional => 1,
+            },
+            zone_map => {
+                type    => t('HashRef'),
+                default => sub { {} },
+            },
+            locale => {
+                type    => t('Locale'),
+                default => sub {$en_locale},
+            },
+            on_error => {
+                type    => t('OnError'),
+                default => 'undef',
+            },
+            strict => {
+                type    => t('Bool'),
+                default => 0,
+            },
+            debug => {
+                type    => t('Bool'),
+                default => $ENV{DATETIME_FORMAT_STRPTIME_DEBUG},
             },
         },
-        debug => {
-            type    => BOOLEAN,
-            default => $ENV{DATETIME_FORMAT_STRPTIME_DEBUG},
-        },
-    };
+    );
 
     sub new {
         my $class = shift;
-        my %args = validate( @_, $spec );
+        my %args  = $validator->(@_);
 
         if ( $args{locale} && !ref $args{locale} ) {
             $args{locale} = DateTime::Locale->load( $args{locale} )
@@ -70,14 +74,15 @@ use constant PERL_58 => $] < 5.010;
 
         my $self = bless {
             %args,
-            zone_map => $class->_build_zone_map,
+            zone_map => $class->_build_zone_map( $args{zone_map} ),
         }, $class;
 
         # Forces a check that the pattern is valid
         $self->_parser;
 
-        binmode STDERR, ':encoding(UTF-8)'
-            if $self->{debug};
+        if ( $self->{debug} ) {
+            binmode STDERR, ':encoding(UTF-8)' or die $!;
+        }
 
         return $self;
     }
@@ -191,8 +196,16 @@ use constant PERL_58 => $] < 5.010;
         'UTC'    => '+0000',
     );
 
+    for my $i ( map { sprintf( '%02d', $_ ) } 1 .. 12 ) {
+        $zone_map{ '-' . $i } = '-' . $i . '00';
+        $zone_map{ '+' . $i } = '+' . $i . '00';
+    }
+
     sub _build_zone_map {
-        return \%zone_map;
+        return {
+            %zone_map,
+            %{ $_[1] },
+        };
     }
 }
 
@@ -231,12 +244,16 @@ sub parse_datetime {
     # We need to copy the %args here because _munge_args will delete keys in
     # order to turn this into something that can be passed to a DateTime
     # constructor.
-    my ( $constructor, $args ) = $self->_munge_args( {%args} )
-        or return;
+    my ( $constructor, $args, $post_construct )
+        = $self->_munge_args( {%args} );
+    return unless $constructor && $args;
 
     my $dt = try { DateTime->$constructor($args) };
     $self->_our_croak('Parsed values did not produce a valid date')
         unless $dt;
+    if ($post_construct) {
+        $post_construct->($dt);
+    }
     return unless $dt && $self->_check_dt( $dt, \%args );
 
     $dt->set_time_zone( $self->{time_zone} )
@@ -274,7 +291,6 @@ sub _build_parser {
 
     my $regex;
     my @fields;
-    my @postprocess;
 
     while (
         $pattern =~ /
@@ -317,14 +333,15 @@ sub _build_parser {
     }
 
     return {
-        regex  => qr/\A\s*$regex/,
+        regex =>
+            ( $self->{strict} ? qr/(?:\A|\b)$regex(?:\b|\Z)/ : qr/$regex/ ),
         fields => \@fields,
     };
 }
 
 {
-    my $d                 = qr/(?:[0-9])/;
-    my $one_or_two_digits = qr/[0-9 ]?$d/;
+    my $digit             = qr/(?:[0-9])/;
+    my $one_or_two_digits = qr/[0-9 ]?$digit/;
 
     # These patterns are all locale-independent. There are a few that depend
     # on the locale, and must be re-calculated for each new parser object.
@@ -345,7 +362,7 @@ sub _build_parser {
             field => 'iso_week_year_100',
         },
         G => {
-            regex => qr/$d{4}/,
+            regex => qr/$digit{4}/,
             field => 'iso_week_year',
         },
         H => {
@@ -357,7 +374,7 @@ sub _build_parser {
             field => 'hour_12',
         },
         j => {
-            regex => qr/$d{1,3}/,
+            regex => qr/$digit{1,3}/,
             field => 'day_of_year',
         },
         m => {
@@ -376,7 +393,7 @@ sub _build_parser {
             field => 'time_zone_name',
         },
         s => {
-            regex => qr/$d+/,
+            regex => qr/$digit+/,
             field => 'epoch',
         },
         S => {
@@ -404,15 +421,15 @@ sub _build_parser {
             field => 'year_100',
         },
         Y => {
-            regex => qr/$d{4}/,
+            regex => qr/$digit{4}/,
             field => 'year',
         },
         z => {
-            regex => qr/[+-]$d{4}/,
+            regex => qr/(?:Z|[+-]$digit{2}(?:[:]?$digit{2})?)/,
             field => 'time_zone_offset',
         },
         Z => {
-            regex => qr/[a-zA-Z]{1,6}/,
+            regex => qr/[a-zA-Z]{1,6}|[\-\+]$digit{2}/,
             field => 'time_zone_abbreviation',
         },
     );
@@ -542,6 +559,7 @@ sub _token_re_for {
         year_100
     );
 
+    ## no critic (Subroutines::ProhibitExcessComplexity)
     sub _munge_args {
         my $self = shift;
         my $args = shift;
@@ -588,8 +606,16 @@ sub _token_re_for {
         }
 
         if ( $args->{time_zone_offset} ) {
-            $args->{time_zone} = DateTime::TimeZone->new(
-                name => $args->{time_zone_offset} );
+            my $offset = $args->{time_zone_offset};
+
+            if ( $offset eq 'Z' ) {
+                $offset = '+0000';
+            }
+            elsif ( $offset =~ /^[+-][0-9]{2}$/ ) {
+                $offset .= '00';
+            }
+
+            $args->{time_zone} = DateTime::TimeZone->new( name => $offset );
         }
 
         if ( defined $args->{time_zone_abbreviation} ) {
@@ -640,24 +666,39 @@ sub _token_re_for {
             $args->{$k} =~ s/^\s+//;
         }
 
-        # If we parsed "12345" we treat it as "123450000" but if we parsed
-        # "000123456" we treat it as 123,456 nanoseconds. This is all a bit
-        # weird and confusing but it matches how this module has always
-        # worked.
-        $args->{nanosecond} *= 10**( 9 - length $args->{nanosecond} )
-            if defined $args->{nanosecond} && length $args->{nanosecond} != 9;
+        if ( defined $args->{nanosecond} ) {
+
+            # If we parsed "12345" we treat it as "123450000" but if we parsed
+            # "000123456" we treat it as 123,456 nanoseconds. This is all a bit
+            # weird and confusing but it matches how this module has always
+            # worked.
+            $args->{nanosecond} *= 10**( 9 - length $args->{nanosecond} )
+                if length $args->{nanosecond} != 9;
+
+            # If we parsed 000000123 we want to turn this into a number.
+            $args->{nanosecond} += 0;
+        }
 
         for my $k (qw( year month day )) {
             $args->{$k} = 1 unless defined $args->{$k};
         }
 
         if ( defined $args->{epoch} ) {
-            $args->{epoch} .= q{.} . $args->{nanosecond}
-                if defined $args->{nanosecond};
+
+            # We don't want to pass a non-integer epoch value since that gets
+            # truncated as of DateTime 1.22. Instead, we'll set the nanosecond
+            # to parsed value after constructing the object. This is a hack,
+            # but it's the best I can come up with.
+            my $post_construct;
+            if ( my $nano = $args->{nanosecond} ) {
+                $post_construct = sub { $_[0]->set( nanosecond => $nano ) };
+            }
+
             delete @{$args}{
                 qw( day_of_year year month day hour minute second nanosecond )
             };
-            return ( 'from_epoch', $args );
+
+            return ( 'from_epoch', $args, $post_construct );
         }
         elsif ( $args->{day_of_year} ) {
             delete @{$args}{qw( epoch month day )};
@@ -668,6 +709,7 @@ sub _token_re_for {
     }
 }
 
+## no critic (Subroutines::ProhibitExcessComplexity)
 sub _check_dt {
     my $self = shift;
     my $dt   = shift;
@@ -836,6 +878,7 @@ sub _check_dt {
 
     return 1;
 }
+## use critic
 
 sub pattern {
     my $self = shift;
@@ -1002,13 +1045,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 DateTime::Format::Strptime - Parse and format strp and strf time patterns
 
 =head1 VERSION
 
-version 1.63
+version 1.72
 
 =head1 SYNOPSIS
 
@@ -1034,9 +1079,6 @@ version 1.63
         on_error  => 'croak',
     );
 
-    # Will throw an exception
-    $newpattern = $strp->pattern('%Q');
-
     # Do something else when things go wrong:
     my $strp = DateTime::Format::Strptime->new(
         pattern   => '%T',
@@ -1052,8 +1094,6 @@ reverse of C<strftime(3)>, for C<DateTime>. While C<strftime> takes a
 C<DateTime> and a pattern and returns a string, C<strptime> takes a string and
 a pattern and returns the C<DateTime> object associated.
 
-=encoding UTF-8
-
 =for Pod::Coverage parse_duration format_duration
 
 =head1 METHODS
@@ -1070,10 +1110,43 @@ This methods creates a new object. It accepts the following arguments:
 
 This is the pattern to use for parsing. This is required.
 
+=item * strict
+
+This is a boolean which disables or enables strict matching mode.
+
+By default, this module turns your pattern into a regex that will match
+anywhere in a string. So given the pattern C<%Y%m%d%H%M%S> it will match a
+string like C<20161214233712Z>. However, this also means that a this pattern
+will match B<any> string that contains 14 or more numbers! This behavior can
+be very surprising.
+
+If you enable strict mode, then the generated regex is wrapped in boundary
+checks of the form C</(?:\A|\b)...(?:\b|\z_/)>. These checks ensure that the
+pattern will only match when at the beginning or end of a string, or when it
+is separated by other text with a word boundary (C<\w> versus C<\W>).
+
+By default, strict mode is off. This is done for backwards
+compatibility. Future releases may turn it on by default, as it produces less
+surprising behavior in many cases.
+
+Because the default may change in the future, B<< you are strongly encouraged
+to explicitly set this when constructing all C<DateTime::Format::Strptime>
+objects >>.
+
 =item * time_zone
 
 The default time zone to use for objects returned from parsing.
 
+=item * zone_map
+
+Some time zone abbreviations are ambiguous (e.g. PST, EST, EDT). By default,
+the parser will die when it parses an ambiguous abbreviation. You may specify
+a C<zone_map> parameter as a hashref to map zone abbreviations however you like:
+
+    zone_map => { PST => '-0800', EST => '-0600' }
+
+Note that you can also override non-ambiguous mappings if you want to as well.
+
 =item * locale
 
 The locale to use for objects returned from parsing.
@@ -1165,14 +1238,18 @@ The % character.
 
 =item * %a or %A
 
-The weekday name according to the current locale, in abbreviated form or
+The weekday name according to the given locale, in abbreviated form or
 the full name.
 
 =item * %b or %B or %h
 
-The month name according to the current locale, in abbreviated form or
+The month name according to the given locale, in abbreviated form or
 the full name.
 
+=item * %c
+
+The datetime format according to the given locale.
+
 =item * %C
 
 The century number (0-99).
@@ -1228,10 +1305,10 @@ Arbitrary whitespace.
 
 Nanoseconds. For other sub-second values use C<%[number]N>.
 
-=item * %p
+=item * %p or %P
 
-The equivalent of AM or PM according to the locale in use. (See
-L<DateTime::Locale>)
+The equivalent of AM or PM according to the locale in use. See
+L<DateTime::Locale>.
 
 =item * %r
 
@@ -1276,6 +1353,14 @@ The weekday number (0-6) with Sunday = 0.
 The week number with Monday the first day of the week (0-53). The first
 Monday of January is the first day of week 1.
 
+=item * %x
+
+The date format according to the given locale.
+
+=item * %X
+
+The time format according to the given locale.
+
 =item * %y
 
 The year within century (0-99). When a century is not otherwise specified
@@ -1309,13 +1394,6 @@ C<strftime()> method, but can be passed to C<format_datetime()>.
 
 This module was created by Rick Measham.
 
-=head1 BUGS
-
-Please report any bugs or feature requests to
-C<bug-datetime-format-strptime@rt.cpan.org>, or through the web interface at
-L<http://rt.cpan.org>. I will be notified, and then you'll automatically be
-notified of progress on your bug as I make changes.
-
 =head1 SEE ALSO
 
 C<datetime@perl.org> mailing list.
@@ -1324,6 +1402,41 @@ http://datetime.perl.org/
 
 L<perl>, L<DateTime>, L<DateTime::TimeZone>, L<DateTime::Locale>
 
+=head1 BUGS
+
+Please report any bugs or feature requests to
+C<bug-datetime-format-strptime@rt.cpan.org>, or through the web interface at
+L<http://rt.cpan.org>. I will be notified, and then you'll automatically be
+notified of progress on your bug as I make changes.
+
+Bugs may be submitted at L<https://github.com/houseabsolute/DateTime-Format-Strptime/issues>.
+
+There is a mailing list available for users of this distribution,
+L<mailto:datetime@perl.org>.
+
+I am also usually active on IRC as 'autarch' on C<irc://irc.perl.org>.
+
+=head1 SOURCE
+
+The source code repository for DateTime-Format-Strptime can be found at L<https://github.com/houseabsolute/DateTime-Format-Strptime>.
+
+=head1 DONATIONS
+
+If you'd like to thank me for the work I've done on this module, please
+consider making a "donation" to me via PayPal. I spend a lot of free time
+creating free software, and would appreciate any support you'd care to offer.
+
+Please note that B<I am not suggesting that you must do this> in order for me
+to continue working on this particular software. I will continue to do so,
+inasmuch as I have in the past, for as long as it interests me.
+
+Similarly, a donation made in this way will probably not make me work on this
+software much more, unless I get so many donations that I can consider working
+on free software full time (let's all have a chuckle at that together).
+
+To donate, log into PayPal and send money to autarch@urth.org, or use the
+button at L<http://www.urth.org/~autarch/fs-donation.html>.
+
 =head1 AUTHORS
 
 =over 4
@@ -1338,18 +1451,35 @@ Rick Measham <rickm@cpan.org>
 
 =back
 
-=head1 CONTRIBUTOR
+=head1 CONTRIBUTORS
 
-=for stopwords D. Ilmari Mannsåker
+=for stopwords Christian Hansen D. Ilmari Mannsåker key-amb
+
+=over 4
+
+=item *
+
+Christian Hansen <chansen@cpan.org>
+
+=item *
 
 D. Ilmari Mannsåker <ilmari.mannsaker@net-a-porter.com>
 
+=item *
+
+key-amb <yasutake.kiyoshi@gmail.com>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by Dave Rolsky.
+This software is Copyright (c) 2015 - 2017 by Dave Rolsky.
 
 This is free software, licensed under:
 
   The Artistic License 2.0 (GPL Compatible)
 
+The full text of the license can be found in the
+F<LICENSE> file included with this distribution.
+
 =cut
